﻿﻿'''
/////////////////////////////////////////////////////////////////////////////////////////////////////////

Blender PolyGroup Maker v0.0.11

Author: Christophe Leblanc. 2018 < christopheleblanc@gmx.com >

Last Update: 14/12/2018

/////////////////////////////////////////////////////////////////////////////////////////////////////////

Description:

	Blender PolyGroup Maker is an Add-on for the Blender 3d modelling software.
	It will help blender artists to create groups of duplicated objects positioned in form of a 
	regular polygon. It add a panel on the "Properties > Object" sector to control some options
	to facilitates the creation and the edition of the PolyGroup.

	How it works:
	To create a "PolyGroup", this Add-on takes the active object and create N copies to be 
	positioned around it, at a given distance. This original object is then virtually removed from the 
	scene and replaced by a group node containing the copies, it can be re-added to the scene later.

Instructions:

	Create a PolyGroup:
		->Select an object
		->Go to "Properties > Object > PolyGroup Maker"
		->Enter creation options and then push "Generate"

	Modify a PolyGroup:
		->Select the group node
		->Go to "Properties > Object > PolyGroup Maker"

	Delete a PolyGroup:
		->Select the group node
		->Go to "Properties > Object > PolyGroup Maker"
		:   ->Push "Destroy" to delete a PolyGroup
		:   ->Push "Cancel" to delete a PolyGroup and re-add the original object on the scene

License:
	
	This script is distributed under the GPL License

	# ##### BEGIN GPL LICENSE BLOCK #####

		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software Foundation,
		Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
		
	# ##### END GPL LICENSE BLOCK #####


/////////////////////////////////////////////////////////////////////////////////////////////////////////
'''


import bpy, bmesh
from bpy.props import (StringProperty,
					   BoolProperty,
					   IntProperty,
					   FloatProperty,
					   EnumProperty,
					   PointerProperty,
					   CollectionProperty,
					   FloatVectorProperty
					   )
from bpy.types import (Panel,
					   Operator,
					   PropertyGroup,
					   UIList,
					   Menu
					   )
import math
import mathutils
from mathutils import (Vector,
					   Euler,
					   )
import fnmatch
import re
from enum import Enum

#SOME CONSTANTS
SHOW_DEBUG = True #Show some debug text in the blender console
SHOW_REINIT_BUTTON = False #Show some debug text in the blender console

TRACKENUMSTR = "Tracking Axis " #String used by GUI ComboBox
PANESPLIT_PERC_A = 0.90 #Common percentage used by UI split()

# class Lang 
# Some texts used by the Add-on
class Lang:
	def __init__(self):
		self.err_nodata = "No data stored"
		self.err_nosource = "No source object"
		self.err_notcomp_0 = "Object \""
		self.err_notcomp_1 = "\" type \""
		self.err_notcomp_2 = "\" is not compatible with this Add-on."
		self.err_nomat = "No material"


# Enum of destruction modes
class DestructionMode(Enum):
	CANCEL = 0
	DESTROY = 1



#Active object follower may be useless if we can acces the current PolyGroup by context.object
class ActiveObjectFollower:

	def __init__(self):
		self.current = None
		self.last = None
		self.changed = False
		self.onNewActive = None

	#Add a callback function to the event "on new active object"
	def setOnNewActive(self, func):
		self.onNewActive = func

	def treatCallback(self):
		if self.onNewActive != None:
			self.onNewActive()

	def update(self, sendCallback = None):

		#Function params defaults
		if sendCallback is None:
			sendCallback = True


		#Check if the current context is the compatible context and has the attribute "active_object"
		if hasattr(bpy.context, 'active_object'):
			active = bpy.context.active_object
		else:
			active = None
		

		#Test object equality
		if not equalObjects(active, self.current):
			self.last = self.current
			self.current = active
			self.changed = True
			
			#On new active_object
			if sendCallback:
				self.treatCallback()
				
		else:
			if self.changed is True:
				self.changed = False
				
		

	def setNone(self):
		self.last = None
		self.current = None
		self.changed = False


class ActivePolyGroupFollower(ActiveObjectFollower):
	def __init__(self):
		super().__init__()
		self.isPolyGroup = False
		self.isCompatible = False
		self.propGroup = None
		self.propGroupIdx = -1

	def update(self):

		super().update(False)

		if self.changed:

			if self.current is not None:

				self.isPolyGroup = self.current.pgprops.ispolygroup

				if self.isPolyGroup:
					self.isCompatible = False

				else:
					self.isCompatible = testCompatibility(self.current)
			else:
				self.isPolyGroup = False
				self.isCompatible = False

			self.treatCallback()

		#Nothing if not changed

	def setNone(self):
		self.isPolyGroup = False
		self.isCompatible = False
		super().setNone()


#Class to represent PolyGroups that are waiting their own destruction when destroying/canceling the PolyGroup
class PendingDestructionPolyGroup:
	def __init__(self, obj, mode):
		self.object = obj
		self.mode = mode

#ListenedObject is the object in which we store a Blender object. 
#Since the Blender API does not dispose any "Property" class to store a pointer to an object, 
#we store the name separately to keep listen on any change on the scene.
class ListenedObject:
	def __init__(self, obj):
		self.object = obj
		self.name = obj.name

class ListenedPolyGroup:
	def __init__(self, lParent, lChildren):
		self.listenedParent = lParent
		self.listenedChildren = lChildren

	def getChildIndexOf(self, obj):
		lisLen = len(self.listenedChildren)
		for i in range(lisLen):
			lisPG = self.listenedChildren[i]
			if lisPG.object == obj:
				return i
		return -1

	def removeChild(self, obj):
		idx = self.getChildIndexOf(obj)
		if idx is not None:
			self.listenedChildren.pop(idx)
		

#Init the Add-on

lang = Lang()

activeFollower = ActivePolyGroupFollower()

tmpSourceObj = None

pre_sels = None
#array(type: objects): stored_presels :used to store/restore the user selection when calling Select Operator
stored_presels = []

pendingDestuctionPolyGroups = []

addonListenedPolygroups = []

matInputEmpty = True
matInputGood = True

#boolean to enabled updating or not when processing a deletion
allowUpdate = True

def onNewActive():

	global activeFollower

	return None

#activeFollower.setOnNewActive(onNewActive)
					   
def instanceName(self):

	return self.__class__.__name__


def stripTrailingNumber(s):
    m = re.search(r'\.(\d{3})$', s)
    return s[0:-4] if m else s

#Create an UniqueName of blender object
#Source: https://blender.stackexchange.com/questions/42613/auto-naming-of-items
def createUniqueName(collection, base_name):
    base_name = stripTrailingNumber(base_name)
    count = 1
    name = base_name

    while collection.get(name):
        name = "%s.%03d" % (base_name, count)
        count += 1      
    return name



#Blender types: ‘MESH’, ‘CURVE’, ‘SURFACE’, ‘META’, ‘FONT’, ‘ARMATURE’, ‘LATTICE’, ‘EMPTY’, ‘CAMERA’, ‘LAMP’, ‘SPEAKER’
def testCompatibility(obj):

	otype = obj.type
	res = None
	if otype == 'MESH':
		res = True
	else:
		res = False
	return res

def isDeleted(o): 

	return not (o.name in bpy.data.objects)

#Capture the selection in a global temporary array and deselect it
def captureSelection():
	global pre_sels

	pre_sels = getSelectedObjects()
	for sel in pre_sels:
		sel.select = False

#Reselect the selection
def releaseSelection():
	global pre_sels

	#Restore selection
	for sel in pre_sels:
		sel.select = True

	pre_sels.clear()


def resetAddon():
	#Do nothing
	return None


#############

#Find the angle of a segment from point 1 (x1, y1) to point 2 (x2, y2)
#Calculates the angle of the vector from the first point to the second point.
def betweenPoints2D(x1, y1, x2, y2):

	return math.atan2(y2 - y1, x2 - x1)


#Find the angle of the segment between object1 and object2
#obj1 = Object to face, obj2 = Object to rotate
#return angle in radians
def rotateObjectToCenter(obj):

	scene = bpy.context.scene
	loc = obj.location
	angle = betweenPoints2D(loc.x, loc.y, 0, 0) #radians
	angle += scene.trackingfaceangle

	return angle #radians


#Calculate and apply the angle of rotation Z of a child
def doChildTrackedRotationZ(polyObject, polyProps, child, childRotation):

	childRotation.z = rotateObjectToCenter(child)
	childRotation.z += polyProps.trackingfaceangle
	childRotation.z += math.radians(polyProps.rotationz)


#Calculate and apply the angle of rotation (Z) of a Child
def doChildRotationZ(polyObject, polyProps, child, childRotation):
	if polyProps.tracking:

		doChildTrackedRotationZ(polyObject, polyProps, child, childRotation)

	else:

		childRotation.z = math.radians(polyProps.rotationz)


#Calculate and apply the angle of rotation (Z) of a PolyGroup
def doGroupRotationZ(polyObject, polyProps):
	if polyProps.tracking:

		for item in polyProps.children:
			obj = bpy.data.objects[item.name]
			rot = obj.rotation_euler

			doChildTrackedRotationZ(polyObject, polyProps, obj, rot)

	else:

		for item in polyProps.children:
			obj = bpy.data.objects[item.name]
			obj.rotation_euler.z = math.radians(polyProps.rotationz)


#Calculate and apply the rotation vector of a PolyGroup
def doGroupRotation(polyObject, polyProps):

	if polyProps.tracking:

		for item in polyProps.children:
			obj = bpy.data.objects[item.name]

			rot = obj.rotation_euler
			rot.x = math.radians(polyProps.rotationx)
			rot.y = math.radians(polyProps.rotationy)

			doChildTrackedRotationZ(polyObject, polyProps, obj, rot)
			

	else:

		for item in polyProps.children:
			obj = bpy.data.objects[item.name]

			rot = obj.rotation_euler
			rot.x = math.radians(polyProps.rotationx)
			rot.y = math.radians(polyProps.rotationy)
			rot.z = math.radians(polyProps.rotationz)


#Compute the 2D rotation vector of an angle (in radians)
def computeRotationVector(angle):

	#px, py are constants
	px = 1
	py = 0
	
	s = math.sin(angle)
	c = math.cos(angle)
	
	x = px * c - py * s
	y = px * s + py * c
	
	return Vector([x, y])


#Generate a polygon and return it as an array of vectors
def genPolygon(nPoints, startAngle=None, maxAngle=None, angularOffsetValue=None, closed=None):
	
	#Function defaults
	if startAngle is None:
		startAngle = 0
	if maxAngle is None:
		maxAngle = 360
	if angularOffsetValue is None:
		angularOffsetValue = 0
	if closed is None:
		closed = False
		
	#Create new array
	vectors = []
	
	flen = nPoints
	stepAngle = maxAngle / nPoints;
	angularOffset = stepAngle * angularOffsetValue
	
	for i in range(0, flen):

		angle = ((stepAngle * i) + startAngle) + angularOffset
		vec = computeRotationVector(math.radians(angle))
		
		vectors.append(vec)
		
	return vectors
		

def equalObjects(a, b):

	if a is None:
		if b is None:
			return True
		else:
			return False
	else:
		if b is None:
			return False
		else:
			if a == b:
				return True
			else:
				return False

def getListenedPolyGroup(obj):

	global addonListenedPolygroups

	for lisPG in addonListenedPolygroups:
		if lisPG.listenedParent.object == obj:
			return lisPG
	return None

def getListenedPolyGroupIndex(obj):

	global addonListenedPolygroups

	lisLen = len(addonListenedPolygroups)

	for i in range(lisLen):
		lisPG = addonListenedPolygroups[i]
		if lisPG.listenedParent.object == obj:
			return i
	return -1

def removeListenedPolyGroup(obj):

	global addonListenedPolygroups

	idx = getListenedPolyGroupIndex(obj)
	if idx is not -1:
		addonListenedPolygroups.pop(idx)
	
def getSelectedObjects():
	
	objects = bpy.context.scene.objects

	return [ob for ob in objects if ob.select]
	

#Test array of objects equality by comparing names
def selectedPGEquals(arrayA, arrayB):

	lenA = len(arrayA)
	lenB = len(arrayB)
	if lenB != lenA:
		return False
	else:
		for x in range(lenB):
			obj = bpy.data.objects.get(arrayB[x].name)
			if obj is not None:
				if obj.select is False:
					return False
			else:
				return False
		return True


def applyRadius(polyProps):
	
	for item in polyProps.children:

		obj = bpy.data.objects[item.name]
		
		radius = polyProps.radius
		
		vec = item.vec
		
		obj.location.x = vec[0] * radius;
		obj.location.y = vec[1] * radius;


#Transform a PolyGroup without changing the number of children
def applyModsToPolyGroup(polyProps):

	polyObject = bpy.data.objects[polyProps.name]
	
	for item in polyProps.children:
		
		radius = polyProps.radius
		
		vec = item.vec

		obj = bpy.data.objects[item.name]
		
		obj.location.x = vec[0] * radius;
		obj.location.y = vec[1] * radius;

		#Apply children rotation
		rot = obj.rotation_euler
		rot.x = math.radians(polyProps.rotationx)
		rot.y = math.radians(polyProps.rotationy)

		doChildRotationZ(polyObject, polyProps, obj, rot)

		#Scale
		obj.scale.x = polyProps.scalex
		obj.scale.y = polyProps.scaley
		obj.scale.z = polyProps.scalez

		
#Remake the polygroup
#Transform a PolyGroup and change the number of children at same time
def remakePolyGroup(polyProps):
	
	sels = getSelectedObjects()
	
	
	for sel in sels:
		sel.select = False
	
	#Retrieve the PolyGroup object
	groupObject = bpy.data.objects[polyProps.name]
	pos = groupObject.location

	userRadius = polyProps.radius
	nPoints = polyProps.npoints
	startAngle = polyProps.startangle
	endAngle = polyProps.endangle
	angOffset = polyProps.angularoffset

	polyLen = len(polyProps.children)
	
	lisPolyGroup = getListenedPolyGroup(groupObject)
	
	#Trim the list if the number of points is less than the older
	if polyLen > nPoints:
		
		dif = polyLen - nPoints

		for i in range(dif):
			idx = polyLen - ( i + 1 )
			objName = polyProps.children[idx].name
			dataObjs = bpy.data.objects
			obj = dataObjs[objName]
			polyProps.children.remove(idx)
			lisPolyGroup.removeChild(obj)
			dataObjs.remove(dataObjs[objName], True)
		
	
		polyLen = len(polyProps.children) #update polyLen
	
	coords = genPolygon(nPoints, startAngle, endAngle, angOffset, False)
	
	sourceObj = bpy.data.objects[polyProps.sourcename]

	i = 0
	for coord in coords:

		curObj = None
		
		loc_x = coord.x * userRadius
		loc_y = coord.y * userRadius
		loc_z = 0 #location Z inherited from PolyGroup Node = 0

		if i < polyLen:

			item = polyProps.children[i]

			item.vec[0] = coord.x
			item.vec[1] = coord.y

			curObj = bpy.data.objects[item.name]

		else:

			#Create duplicate
			dup = duplicateObject(sourceObj)
			
			dup.parent = groupObject

			#if polyProps.gotmaterials:

				#mat = bpy.data.materials[polyProps.material]
				#if mat is not None:
					#setMaterial(dup, mat)
					#polyProps.materials

			#setMaterialLinkObject(child.object)
			
			addObjectToList(polyProps, dup, coord)

			lisChild = ListenedObject(dup)
			lisPolyGroup.listenedChildren.append(lisChild)

			curObj = dup


		setLocation(curObj, loc_x, loc_y, loc_z)

		#Apply children rotation
		rot = curObj.rotation_euler
		rot.x = math.radians(polyProps.rotationx)
		rot.y = math.radians(polyProps.rotationy)

		doChildRotationZ(groupObject, polyProps, curObj, rot)

		curObj.scale.x = polyProps.scalex
		curObj.scale.y = polyProps.scaley
		curObj.scale.z = polyProps.scalez

		i += 1
		
	for sel in sels:
		sel.select = True
		
		
	#Prevent UIListIndex to be out of range
	max = nPoints - 1
	if bpy.context.scene.list_1_index > max:
		bpy.context.scene.list_1_index = max


	polyProps.npoints_prev = polyProps.npoints

		
#delPGChildren: Delete PolyGroup children from the current scene
# This is used on group destruction, but not on cancel
def delPGChildren(polyProp):
	
	# deselect all
	#bpy.ops.object.select_all(action='DESELECT')

	childrenItems = polyProp.children

	#select all stored nodes
	if len(childrenItems) > 0:
		for childItem in childrenItems:
			childObj = bpy.data.objects[childItem.name]
			if not isDeleted(childObj):
				childObj.select = True


		
		groupObj = bpy.data.objects[polyProp.name]
		if groupObj is not None:
			if not isDeleted(groupObj):
				groupObj.select = True

	#delete them
	bpy.ops.object.delete()


	
#delPGSource: Delete PolyGroup children from the current scene
# Used on group destruction
def delTMPPGSource():
	
	global tmpSourceObj
	
	tmpSourceObj.select = True
			
	#delete
	bpy.ops.object.delete()

	
#delProc: Delete procedure of the group
# Used on group destruction
def delProc(polyProp):
	
	#un-select user selection
	next_act = bpy.context.scene.objects.active
	bpy.context.scene.objects.active = None

	sels = getSelectedObjects()
	next_sels = []
	for sel in sels:
		sel.select = False
		if not isDeleted(sel):
			next_sels.append(sel)
		
	delPGChildren(polyProp)
	
	#re-select user selection
	for sel in next_sels:
		sel.select = True
			
	#reinit add-on variables

	scene = bpy.context.scene

	scene.listselected = False
	stored_presels.clear()
	scene.list_1.clear()


	

def setMaterial(obj, mat):

	#Check all is ok
	if obj and mat:

		#If a material is already assigned (material slots is not empty)
		if obj.material_slots:
			#Assign the material to the first slot
			obj.material_slots[0].material = mat
		#If no material is assigned
		else:
			#Create a new material (link/slot) for the object
			obj.data.materials.append(mat)

	#for mat_slot in obj.material_slots:
	#    mat = mat_slot.material
	#    mat_slot.link = 'OBJECT'
	#    mat_slot.material = mat


def setTexture(mat, tex):

	if mat.texture_slots and mat.texture_slots[0] is not None:

		mat.texture_slots[0].texture = tex

	else:

		slot = mat.texture_slots.add()
		slot.texture = tex

	return None
	

def setLocation(obj, loc_x, loc_y, loc_z):
	loc = obj.location
	loc.x = loc_x
	loc.y = loc_y
	loc.z = loc_z


#Translate the "Tracking" enumeration to an exploitable angle (in radians)
def trackEToA(enum):

	res = 0
	
	if enum == "1":
		res = 90
	elif enum == "2":
		res = 180
	elif enum == "3":
		res = 270
	else:#X
		res = 0

	return math.radians(res)



def addObjectToList(objectProps, obj, vec):
	
	#item = bpy.context.scene.list_1.add()
	item = objectProps.children.add()
	item.name = obj.name
	item.vec[0] = vec.x
	item.vec[1] = vec.y
	

#Duplicate a blender object (type= "MESH")
def duplicateObject(source):
		
	# make a duplicate only of the mesh
	mesh_dup = source.data.copy()
		
	# make a new object, and reference the duplicated mesh.

	source_dup = bpy.data.objects.new(source.name, mesh_dup)
		
	#the object must be added to the scene
	bpy.context.scene.objects.link(source_dup)

	#if source_dup.material_slots:
		#for mat in source_dup.material_slots:
			#print("mat name: " + mat.name)
			#source_dup.data.materials.append(mat.material)
		
	# prove that the object is created by offsetting its location.
	#solid_duplicate.location = solid.location + Vector((2,0,0))
		
	#Test rotation
	#solid_duplicate.rotation_euler = Euler((0.3, 0.3, 0.4), 'XYZ')
		
	return source_dup



def getStoredPolyGroupIndex(objName):
	arr = bpy.context.scene.polygroups
	for i in range(len(arr)):
		if arr[i].name == objName:
			return i
	return -1


'''
/////////////////////////////////////////////////////////////////////////////////////////////////////////


#Blender properties update callbacks


/////////////////////////////////////////////////////////////////////////////////////////////////////////
'''


def updateVisibility():

	print("update visible")

	
def updateRadius(self, context):

	currObject = context.object
	polyProps = currObject.pgprops
	
	if polyProps.ispolygroup:
		applyRadius(polyProps)
		
	
# Property callback used when changing the number of children of a PolyGroup
# Note: Unlike other properties callbacks, this one checks the existence of the context object to prevent "NoneType" error
# when changing this property on update. The callback function "remakePolyGroup()" should be called from inside in that case.
def updateNPoints(self, context):
	
	currObject = context.object

	#Test the existence of the context object
	if currObject is not None:
		polyProps = currObject.pgprops
		
		if polyProps.ispolygroup:
			remakePolyGroup(polyProps)
	   

def updateStartAngle(self, context):

	currObject = context.object
	polyProps = currObject.pgprops
	
	if polyProps.ispolygroup:
		remakePolyGroup(polyProps)
	

def updateEndAngle(self, context):

	polyProps = context.object.pgprops
	
	if polyProps.ispolygroup:
		remakePolyGroup(polyProps)

def updateAngularOffset(self, context):

	polyProps = context.object.pgprops
	
	if polyProps.ispolygroup:
		remakePolyGroup(polyProps)

def updateScale(self, context):

	polyProps = context.object.pgprops
	
	if polyProps.ispolygroup:

		for item in polyProps.children:
			obj = bpy.data.objects[item.name]
			obj.scale.x = polyProps.scalex
			obj.scale.y = polyProps.scaley
			obj.scale.z = polyProps.scalez

def updateRotation(self, context):

	polyProps = context.object.pgprops
	
	if polyProps.ispolygroup:

		for item in polyProps.children:
			obj = bpy.data.objects[item.name]
			obj.rotation_euler.x = math.radians(polyProps.rotationx)
			obj.rotation_euler.y = math.radians(polyProps.rotationy)

def updateRotationZ(self, context):

	currObject = context.object
	polyProps = currObject.pgprops
	
	if polyProps.ispolygroup:

		doGroupRotationZ(currObject, polyProps)




def updateTracking(self, context):

	currObject = context.object
	polyProps = currObject.pgprops
	
	if polyProps.ispolygroup:

		#tracking = polyProps.tracking
		#polyProps.tracking = not tracking
		doGroupRotation(currObject, polyProps)


def updateTrackingFace(self, context):

	currObject = context.object
	polyProps = currObject.pgprops
	
	if polyProps.ispolygroup:

		if polyProps.tracking:

			#Store the selected String value as number (in radians)
			polyProps.trackingfaceangle = trackEToA(polyProps.trackingface)

		doGroupRotationZ(currObject, polyProps)


def updateObjListIdx(self, context):

	return None


def updateMatInput(self, context):

	global matInputEmpty, matInputGood

	matin = bpy.context.scene.materialinput
	if len(matin) > 0:
		mat = bpy.data.materials.get(matin)
		if mat is not None:
			
			matInputGood = True
		else:
			matInputGood = False
		matInputEmpty = False
	else:
		matInputGood = False
		matInputEmpty = True

	return None

class GenerateOperator(bpy.types.Operator):
	"""Generate a group of duplicates to form a polygon"""
	bl_idname = "object.generate_operator"
	bl_label = "Generate"

	@classmethod
	def poll(cls, context):
		return context.active_object is not None

	def execute(self, context):

		global addonListenedPolygroups
		
		scene = context.scene
		
		sourceObj = context.active_object
			
		if sourceObj is not None:

			if len(bpy.context.scene.list_1) > 0:
				bpy.context.scene.list_1.clear()
			
			#sels = getSelectedObjects()
			#for sel in sels:
				#sel.select = False
			
			pos = sourceObj.location
		
			radius = bpy.context.scene.radius
			nPoints = bpy.context.scene.npoints
			startAngle = bpy.context.scene.startangle
			endAngle = bpy.context.scene.endangle
			angOffset = bpy.context.scene.angularoffset
			
			coords = genPolygon(nPoints, startAngle, endAngle, angOffset, True)
			
			#Create a new group object
			newgroupName = createUniqueName(bpy.data.objects, "Group")
			newgroup = bpy.data.objects.new(newgroupName, None)
			newgroup.dupli_type = 'GROUP'
			newgroup.location = pos

			polyProps = newgroup.pgprops
			polyProps.ispolygroup = True
			polyProps.radius = radius
			polyProps.npoints = nPoints
			polyProps.npoints_prev = nPoints
			polyProps.startangle = startAngle
			polyProps.endangle = endAngle
			polyProps.created = True
			polyProps.name = newgroupName
			polyProps.sourcename = sourceObj.name

			sourceNMats = 0
			if sourceObj.material_slots:
					for mat in sourceObj.material_slots:
						sourceNMats += 1
			if sourceNMats > 0:

				if sourceObj.material_slots:
					for mat in sourceObj.material_slots:
						newMat = polyProps.materials.add()
						newMat.name = mat.name

				polyProps.gotmaterials = True
			else:
				polyProps.gotmaterials = False


			scene.objects.link(newgroup)
			scene.update() #necessary??
			
			tmpLisChildren = []
			
			#Create duplicates of the source object for each coordinate of the polygon
			for coord in coords:
				
				dup = duplicateObject(sourceObj)

				#Set Location
				setLocation(dup, pos.x + (coord.x * radius), pos.y + (coord.y * radius), 0)
				
				dup.parent = newgroup
				
				#dup.scale.x = bpy.context.scene.scalex
				#dup.scale.y = bpy.context.scene.scaley
				#dup.scale.z = bpy.context.scene.scalez

				#setMaterialLinkObject(child.object)
				
				addObjectToList(newgroup.pgprops, dup, coord)

				lisChild = ListenedObject(dup)
				tmpLisChildren.append(lisChild)
				

			scenePG = bpy.context.scene.polygroups.add()
			scenePG.name = newgroup.name

			#Add to active polygroups
			lisParent = ListenedObject(newgroup)
			lisChildren = tmpLisChildren
			lisPolyGroup = ListenedPolyGroup(lisParent, lisChildren)
			addonListenedPolygroups.append(lisPolyGroup)
			
			if bpy.context.scene.gotpolygroups is False:
				bpy.context.scene.gotpolygroups = True

			
			#Unlink selection (unlink() instead of delete() to keep a trace of the object)
			scene.objects.unlink(sourceObj)
			#sourceObj.select = False
			
			#Auto select/activate the group
			newgroup.select = True
			bpy.context.scene.objects.active = newgroup
			
			
			bpy.context.scene.npoints_prev = bpy.context.scene.npoints
			
			if SHOW_DEBUG:
				print("Generate Operator Finished") #debug
			
			
		return {'FINISHED'}
	

	
class ToZeroOperator(bpy.types.Operator):
	"""Tooltip"""
	bl_idname = "object.tozero_operator"
	bl_label = "Reinit Position"

	@classmethod
	def poll(cls, context):
		return context.active_object is not None

	def execute(self, context):
		
		act_obj = context.active_object
			
		if(act_obj is not None):
			act_obj.location = Vector([0,0,0])
			act_obj.rotation_euler = Euler((0, 0, 0), 'XYZ')
			act_obj.scale = (1, 1, 1)
		
		return {'FINISHED'}
	

class SelectChildrenOperator(bpy.types.Operator):
	"""Select/Deselect the children of the group"""
	bl_idname = "object.selectchildren_operator"
	bl_label = "Select children"

	@classmethod
	def poll(cls, context):
		return context.active_object is not None

	def execute(self, context):

		currObject = context.object
		polyProps = currObject.pgprops
		
		if polyProps.ispolygroup:
			
			if bpy.context.scene.listselected:
				
				#unselect
				for obj in stored_presels:
					obj.select = True

				selectedsPG = bpy.context.scene.selectedpolygroup
				if len(selectedsPG)  > 0:
					selectedsPG.clear()

			else:
				pre_sels = getSelectedObjects()

				for obj in pre_sels:
					obj.select = False
					stored_presels.append(obj)


				#Store the addon selected polygroup
				selectedsPG = bpy.context.scene.selectedpolygroup
				if len(selectedsPG)  > 0:
					selectedsPG.clear()

				selectedPG = selectedsPG.add()
				selectedPG.name = polyProps.name

				for child in polyProps.children:
					selectedChild = selectedPG.children.add()
					selectedChild.name = child.name

			
			val = not bpy.context.scene.listselected
			bpy.context.scene.listselected = val
				
			children = polyProps.children
			for item in children:
				obj = bpy.data.objects[item.name]
				if obj is not None:
					obj.select = val

			#print("Select Operator Executed")
				
		else:
			self.report({'ERROR'}, lang.err_nodata)
		
		
		
		return {'FINISHED'}
	
def prepareTMPPGSource(polyProps):
	global tmpSourceObj
	sourcename = polyProps.sourcename
	tmpSourceObj = bpy.data.objects[sourcename]

def restoreTMPPGSource():
	global tmpSourceObj
	if tmpSourceObj is not None:
		bpy.context.scene.objects.link(tmpSourceObj)
		tmpSourceObj.select = True
		bpy.context.scene.objects.active = tmpSourceObj
		tmpSourceObj = None

#Similar to destroy but this will re-link and select/activate the source object
class CancelOperator(bpy.types.Operator):
	"""Destroy the group and restore the source object."""
	
	bl_idname = "object.cancel_operator"
	bl_label = "Cancel"

	@classmethod
	def poll(cls, context):
		return context.active_object is not None

	def execute(self, context):

		global pendingDestuctionPolyGroups
		
		currObject = context.object
		polyProps = currObject.pgprops
	
		if polyProps.ispolygroup:

			pendingDestuctionPolyGroups.append(PendingDestructionPolyGroup(currObject, DestructionMode.CANCEL))
			
		else:
			self.report({'ERROR'}, lang.err_nodata)

		if SHOW_DEBUG:
			print("Cancel Operator finished")
		
		return {'FINISHED'}
	

#Similar to cancel but this will not re-link the source object to the scene
class DestroyOperator(Operator):
	"""Destroy the group."""

	bl_idname = "object.destroy_operator"
	bl_label = "Destroy"

	def execute(self, context):

		global pendingDestuctionPolyGroups

		currObject = context.object
		polyProps = currObject.pgprops
	
		if polyProps.ispolygroup:
		
			pendingDestuctionPolyGroups.append(PendingDestructionPolyGroup(currObject, DestructionMode.DESTROY))
			
		else:
			self.report({'ERROR'}, lang.err_nodata)

		if SHOW_DEBUG:
			print("Destroy Operator finished")

		return{'FINISHED'}

	

class ToggleMaterialOperator(Operator):
	"""Add a constraint to the group."""

	bl_idname = "object.togglematerial_operator"
	bl_label = "Add/Remove Polygroup material"

	def execute(self, context):

		global matInputGood, matInputEmpty
		
		currObject = context.object
		polyProps = currObject.pgprops
	
		if polyProps.ispolygroup:

			gotmaterials = polyProps.gotmaterials

			if not gotmaterials:

				materialinput = bpy.context.scene.materialinput

				create = -1
				if matInputEmpty:
					create = 0
				else:
					if matInputGood: 
						create = 2
					else:
						create = 1

				if create == 2:
					tmpmat = bpy.data.materials.get(materialinput)
					if tmpmat is None:
						create = 1

				if create == 0:
					matName = createUniqueName(bpy.data.materials, "Material")
					newMat = bpy.data.materials.new(matName)
				elif create == 1:
					matName = createUniqueName(bpy.data.materials, materialinput)
					newMat = bpy.data.materials.new(matName)
				elif create == 2:
					newMat = bpy.data.materials[materialinput]


				newMatProp = polyProps.materials.add()
				newMatProp.name = newMat.name
				
				children = polyProps.children
				for item in children:
					obj = bpy.data.objects[item.name]
					if obj is not None:
						#setMaterial(obj, newMat)
						obj.data.materials.append(newMat)


			else:

				children = polyProps.children
				for item in children:
					obj = bpy.data.objects[item.name]
					if obj is not None:
						#obj.data.materials.pop(0, update_data=True) #pop at index zero
						while obj.data.materials:
							obj.data.materials.pop(0, True)

				for mat in polyProps.materials:
					mat = bpy.data.materials[mat.name]
					bpy.data.materials.remove(mat)

				#bpy.context.scene.update()#???

			bpy.context.scene.materialinput = ""
			polyProps.gotmaterials = not gotmaterials
			
		else:

			self.report({'ERROR'}, lang.err_nodata)
			

		return{'FINISHED'}

	
class ResetRotationOperator(Operator):
	"""Add a new item to the list."""

	bl_idname = "object.resetrotation_operator"
	bl_label = "Search in materials"

	def execute(self, context):

		polyProps = context.object.pgprops

		polyProps.rotationx = 0.0
		polyProps.rotationy = 0.0
		polyProps.rotationz = 0.0

		return{'FINISHED'}


class ResetScaleOperator(Operator):
	"""Add a new item to the list."""

	bl_idname = "object.resetscale_operator"
	bl_label = "Search in materials"

	def execute(self, context):

		polyProps = context.object.pgprops

		polyProps.scalex = 1.0
		polyProps.scaley = 1.0
		polyProps.scalez = 1.0

		return{'FINISHED'}


class ToggleExtend0Operator(bpy.types.Operator):
	bl_idname = 'object.extend0_operator'
	bl_label = 'Show/hide panel extension'

	def execute(self, context):

		tmpProp = bpy.context.scene.extend_0
		bpy.context.scene.extend_0 = not tmpProp

		return {'FINISHED'}


class ToggleExtend1Operator(bpy.types.Operator):
	bl_idname = 'object.extend1_operator'
	bl_label = 'Show/hide panel extension'

	def execute(self, context):

		tmpProp = bpy.context.scene.extend_1
		bpy.context.scene.extend_1 = not tmpProp

		return {'FINISHED'}


class ToggleExtend2Operator(bpy.types.Operator):
	bl_idname = 'object.extend2_operator'
	bl_label = 'Show/hide panel extension'

	def execute(self, context):

		tmpProp = bpy.context.scene.extend_2
		bpy.context.scene.extend_2 = not tmpProp

		return {'FINISHED'}


class ListAItem(PropertyGroup):
	"""Group of properties representing an item in the list."""

	name = StringProperty(
		name = "Name",
		description = "A name for this item",
		default = "Untitled"
		)
		   
	random_prop = StringProperty(
		name = "Any other property you want",
		description = "",
		default = ""
		)

class PolyGroupMaterialPropertyGroup(PropertyGroup):
	"""Group of properties representing an item in the list."""

	name = StringProperty(
		name = "Name",
		description = "A name for this item",
		default = "Untitled"
		)


class PolyGroupChildrenPropertyGroup(PropertyGroup):
	"""Group of properties representing an item in the list."""

	name = StringProperty(
		name = "Name",
		description = "A name for this item",
		default = "Untitled"
		)

	vec = FloatVectorProperty(
		name = "Name",
		min = -1.0,
		max = 1.0,
		default = (0.0, 0.0, 0.0)
		)

class SelectedChildrenPropertyGroup(PropertyGroup):
	"""Group of properties representing an item in the list."""

	name = StringProperty(
		name = "Name",
		description = "A name for this item",
		default = "Untitled"
		)


class SelectedPropertyGroup(PropertyGroup):

	name = StringProperty(
		name = "Name",
		description = "The name of the group",
		default = "Untitled"
		)

	children = CollectionProperty(type = SelectedChildrenPropertyGroup)

class PolyGroupPropertyGroup(PropertyGroup):
	"""Group of properties representing an item in the list."""

	name = StringProperty(
		name = "Name",
		description = "The name of the group",
		default = "Untitled"
		)

	sourcename = StringProperty(
		name = "Source Name",
		description = "The name of the source",
		default = "Untitled"
		)
		   
	children = CollectionProperty(type = PolyGroupChildrenPropertyGroup)

class PolyGroupObjectPropertyGroup(bpy.types.PropertyGroup):
	"""Group of properties representing a polygroup"""

	name = StringProperty(
		name = "Name",
		description = "The name of the group",
		default = "Untitled"
		)
	sourcename = StringProperty(
		name = "Source Name",
		description = "The name of the source",
		default = "Untitled"
		)
	created = BoolProperty(
		name = "Has been created",
		description = "A bool property",
		default = False
		)
	ispolygroup = BoolProperty(
		name = "Is a polygroup",
		description = "A bool property",
		default = False
		)
	radius = FloatProperty(
		name = "Radius",
		description = "A float property",
		default = 1.0,
		min = 0.0,
		max = 999.0,
		update = updateRadius
		)
	npoints = IntProperty(
		name = "Number of points",
		description = "A int property",
		default = 2,
		min = 2,
		max = 100,
		update = updateNPoints
		)
	npoints_prev = IntProperty(
		name = "Number of points",
		description = "A int property",
		default = 2,
		min = 2,
		max = 100
		)
	startangle = FloatProperty(
		name = "Start Angle (deg)",
		description = "The start angle of the polygroup",
		default = 0.0,
		min = -3600.0,
		max = 3600.0,
		update = updateStartAngle
		)
	endangle = FloatProperty(
		name = "End Angle (deg)",
		description = "The end angle of the polygroup",
		default = 360.0,
		min = 0.0,
		max = 360.0,
		update = updateEndAngle
		)
	angularoffset = bpy.props.FloatProperty(
		name = "Offset (-1, 1)",
		description = "The angular offset",
		default = 0,
		min = -1.0,
		max = 1.0,
		update = updateAngularOffset
		)
	children = CollectionProperty(
		type = PolyGroupChildrenPropertyGroup
		)

	materials = CollectionProperty(
		type = PolyGroupMaterialPropertyGroup
		)


	scalex = FloatProperty(
		name = "Scale X",
		description = "Scale X",
		default = 1.0,
		min = 0.0,
		max = 999.0,
		update = updateScale
		)
	scaley = FloatProperty(
		name = "Scale Y",
		description = "Scale Y",
		default = 1.0,
		min = 0.0,
		max = 999.0,
		update = updateScale
		)
	scalez = FloatProperty(
		name = "Scale Z",
		description = "Scale Z",
		default = 1.0,
		min = 0.0,
		max = 999.0,
		update = updateScale
		)
	rotationx = FloatProperty(
		name = "Rotation X",
		description = "Rotation X",
		default = 0.0,
		min = -3600.0,
		max = 3600.0,
		update = updateRotation
		)
	rotationy = FloatProperty(
		name = "Rotation Y",
		description = "Rotation Y",
		default = 0.0,
		min = -3600.0,
		max = 3600.0,
		update = updateRotation
		)
	rotationz = FloatProperty(
		name = "Rotation Z",
		description = "Rotation Z",
		default = 0.0,
		min = -3600.0,
		max = 3600.0,
		update = updateRotationZ
		)
	tracking = BoolProperty(
		name = "Tracking state",
		description = "A bool property",
		default = False,
		update = updateTracking
		)
	trackingface = EnumProperty(
		items= {
			("0", "X", TRACKENUMSTR + "X"),
			("1", "Y", TRACKENUMSTR + "Y"),
			("2", "-X", TRACKENUMSTR + "-X"),
			("3", "-Y", TRACKENUMSTR + "-Y")
			},  
		default = "0",
		name = "Tracking Axis",
		update = updateTrackingFace
		)
	trackingfaceangle = FloatProperty(
		name = "Tracking Face angle",
		description = "The angle of face tracking center",
		default = 0.0,
		min = 0.0,
		max = math.radians(360.0)
		)

	gotmaterials = BoolProperty(
		name = "Has got Material",
		description = "A bool property",
		default = False
		)



class UIList1(UIList):
	"""Demo UIList."""

	def draw_item(self, context, layout, data, item, icon, active_data,
				  active_propname, index):

		# We could write some code to decide which icon to use here...
		custom_icon = 'OBJECT_DATAMODE'

		# Make sure your code supports all 3 layout types
		if self.layout_type in {'DEFAULT', 'COMPACT'}:
			layout.label(item.name, icon = custom_icon)

		elif self.layout_type in {'GRID'}:
			layout.alignment = 'CENTER'
			layout.label("", icon = custom_icon)
	



def dump(obj, text):
	for attr in dir(obj):
		print("%r.%s = %s" % (obj, attr, getattr(obj, attr)))

def getOP(obj, attr):
	return getattr(obj, attr)
	#print(blrna.bl_idname)



class PropLayoutPanel(bpy.types.Panel):
	"""Creates a Panel in the scene context of the properties editor"""
	bl_label = "PolyGroup Maker"
	bl_idname = "SCENE_PT_layout"
	bl_space_type = 'PROPERTIES'
	bl_region_type = 'WINDOW'
	#bl_category = "Tools"
	bl_context = "object"
	
	@classmethod
	def poll(self,context):
		return context.object is not None

	def drawTogPaneTitBar(self, layout, state, opIdName, text):
		srow = layout.row()

		split = srow.split(percentage = PANESPLIT_PERC_A)

		scol = split.column()
		scol.label(text = text)


		icon = None
		if state:
			icon = 'TRIA_DOWN'
		else:
			icon = 'TRIA_LEFT'

		scol = split.column()
		#rown = col.row(align = True)
		scol.operator(opIdName, icon = icon, text = "")

	def draw(self, context):

		layout = self.layout
		scene = context.scene

		currObject = context.object
		polyProps = currObject.pgprops

		activeObject = context.active_object
		activePGProp = None
		
		drawMode = None

		if activeFollower.isPolyGroup is False:
			if activeFollower.isCompatible:
				drawMode = 1
			else:
				drawMode = 0
		else:
			activePGProp = activeFollower.propGroup
			drawMode = 2


		if drawMode != 0:
			
			if SHOW_REINIT_BUTTON:

				#Big button
				row = layout.row()
				row.scale_y = 1.0
				row.operator("object.tozero_operator")
			
			#Separator
			layout.row().separator()

			ccontext = None;
			if drawMode == 1:
				ccontext = scene
			elif drawMode == 2:
				ccontext = polyProps
			
			#Big float
			row = layout.row()
			row.scale_y = 1.0
			row.prop( ccontext, "npoints" )
			
			#Big float
			row = layout.row()
			row.scale_y = 1.0
			row.prop( ccontext, "radius" )

			#Row with two buttons
			row = layout.row()
			row.scale_y = 1.0
			row.prop( ccontext, "startangle" )
			row.prop( ccontext, "endangle" )
			
			#Big float
			row = layout.row()
			row.scale_y = 1.0
			row.prop( ccontext, "angularoffset" )
		
			
		if drawMode == 1:
			
			#Big button
			row = layout.row()
			row.scale_y = 1.0
			row.operator("object.generate_operator")
			
		elif drawMode == 2:
			
			#Separator
			layout.row().separator()

			#Create extendable box "Children"
			extend_2 = bpy.context.scene.extend_2
			box = layout.box()  
			self.drawTogPaneTitBar(box, extend_2, "object.extend2_operator", "Children:")
			
			if extend_2:

				row = box.row()

				row.scale_y = 1.0
				row.template_list("UIList1", "The_List", polyProps, "children", scene, "list_1_index")


			txt = None
			if bpy.context.scene.listselected:
				txt = "Unselect"
			else:
				txt = "Select"
			

			#Row of three buttons button
			row = layout.row()
			row.scale_y = 1.0
			row.operator("object.selectchildren_operator", text = txt)
			row.operator("object.cancel_operator")
			row.operator("object.destroy_operator", text="Destroy")
			
			
			#Separator
			layout.row().separator()

			#Create extendable box "Children Transform"
			box = layout.box()
			extend_1 = bpy.context.scene.extend_1
			self.drawTogPaneTitBar(box, extend_1, "object.extend1_operator", "Children Transform:")

			if extend_1:

				row = box.row()

				#Col left "Rotation"
				col = row.column(align = True)
				srow = col.row(align = True)
				scol = srow.column(align = True)

				scol.label(text="Rotation:")
				scol.prop( polyProps, "rotationx", text="X")
				scol.prop( polyProps, "rotationy", text="Y")
				scol.prop( polyProps, "rotationz", text="Z")
				scol.operator("object.resetrotation_operator", text="Reset")

				#Col right "Scale"
				col = row.column(align = True)
				srow = col.row(align = True)
				scol = srow.column(align = True)

				scol.label(text="Scale:")
				scol.prop( polyProps, "scalex", text="X")
				scol.prop( polyProps, "scaley", text="Y")
				scol.prop( polyProps, "scalez", text="Z")
				scol.operator("object.resetscale_operator", text="Reset")



			#Separator
			layout.row().separator()


			box = layout.box()

			splita = box.split(percentage = PANESPLIT_PERC_A)

			col = splita.column()

			splitb = col.split(percentage = 0.4)

			scol = splitb.column()
			scol.label(text = "Track Center:")

			scol = splitb.column()
			scol.prop( polyProps, "trackingface", text = "")



			icon = None
			if polyProps.tracking:
				icon = 'RESTRICT_VIEW_OFF'
			else:
				icon = 'RESTRICT_VIEW_ON'
				
			scol = splita.column()
			row = scol.row(align = True)
			row.alignment = 'CENTER'
			row.prop( polyProps, "tracking", text = "")
			
			
			
			
			#Separator
			layout.row().separator()


			#Create extendable box
			extend_0 = bpy.context.scene.extend_0
			box = layout.box()  
			self.drawTogPaneTitBar(box, extend_0, "object.extend0_operator", "Material: ")


			if extend_0:

				txta = None
				if not polyProps.gotmaterials:

					if matInputEmpty:
						txta = "Create"
					elif matInputGood:
						txta = "Set"
					else:
						txta = "Create"

				else:
					txta = "Delete"

				# Big render button
				ssrow = box.row()

				ssplit = ssrow.split(percentage=0.85)

				ssscol = ssplit.column(align = True)
				sssrow = ssscol.row()
				#srow.prop( scene, "materialinput", text="")
				sssrow.prop_search(scene, "materialinput", bpy.data, "materials")

				ssscol = ssplit.column(align = True)
				sssrow = ssscol.row()
				sssrow.operator("object.togglematerial_operator", text = txta)



		if drawMode == 0:

			#Draw an error text into the panel
			if activeFollower.current is not None:
				box = layout.box()
				row = box.row()

				txt = lang.err_notcomp_0
				txt += activeFollower.current.name
				txt += lang.err_notcomp_1
				txt += activeFollower.current.type
				txt += lang.err_notcomp_2

				row.label(icon = "ERROR", text = txt)

		row = layout.row()
		row.scale_y = 1
		row.label(text = "")

def scenePGExists(name):
	if bpy.context.scene.gotpolygroups:
		arr = bpy.context.scene.polygroups
		for i in range(len(arr)):
			if arr[i].name == name:
				return True
		return False
	else:
		return False

def checkListSelection():

	global stored_presels

	scene = bpy.context.scene

	if scene.listselected:
		
		selectPGs = scene.selectedpolygroup

		if len(selectPGs)  > 0:

			selectPG = selectPGs[0]

			selGroupName = selectPG.name
			selGroup = bpy.data.objects.get(selGroupName)

			if selGroup is not None:
				
				if scenePGExists(selGroupName):

					groupPolyProps = selGroup.pgprops

					sels = getSelectedObjects()
				
					if not selectedPGEquals(sels, selectPG.children):
						scene.listselected = False
						selectPG.name = ""
						selectPG.children.clear()


def getScenePolyGroupIdx(obj):
	scenePolyGroups = bpy.context.scene.polygroups
	for i in range(len(scenePolyGroups)):
		#print(scenePolyGroups[i].name)
		if scenePolyGroups[i].name == pgObjName:
			return i
	return -1

def checkDestruction():

	global pendingDestuctionPolyGroups, allowUpdate

	penDestroyLen = len(pendingDestuctionPolyGroups)

	if penDestroyLen > 0:
		
		#Disable update to prevent "maximum recursion depth exceeded" on blender objects deletion ("bpy.ops.object.delete()")
		#We suppose that this error happens because "bpy.ops.object.delete()" uses a concurrent thread, so this "update" function continues to test the object
		#to delete while blender is deleting it. So, this error is certainly caused by unsynchronized threads between Blender and this Add-on
		allowUpdate = False

		for node in pendingDestuctionPolyGroups:

			
			pgObject = node.object
			polyProp = pgObject.pgprops

			#pgObject = bpy.data.objects[polyProp.name]
			pgObjName = pgObject.name

			prepareTMPPGSource(polyProp)
			
			#Destroy all
			delProc(polyProp)
			
		
			#Now reinit stored objects
			

			if node.mode == DestructionMode.CANCEL:
				restoreTMPPGSource()
			elif node.mode == DestructionMode.DESTROY:
				delTMPPGSource()

			activeFollower.setNone()

			scenePolyGroups = bpy.context.scene.polygroups
			#print("PGINDEX : " + pgObject.name + " " + str(len(scenePolyGroups)))
			
			pgIdx = getScenePolyGroupIdx(pgObjName)
			
			if pgIdx is not -1:
				bpy.context.scene.polygroups.remove(pgIdx)

			removeListenedPolyGroup(pgObject)

			
		pendingDestuctionPolyGroups.clear()
		bpy.context.scene.gotpolygroups = False
		
		#Re-allow update after deletion is complete to continue using the update function
		allowUpdate = True
		

		#print("scenePolyGroups length " + str(len(scenePolyGroups)))
		#print("pendingDestuctionPolyGroups length " + str(len(pendingDestuctionPolyGroups)))


def preUpdateHandler(scene):

	global activeFollower

	fo = activeFollower
	fo.update()


def checkGroupNameChange(addonPG):
	storedName = addonPG.listenedParent.name
	newName = addonPG.listenedParent.object.name
	if newName is not storedName:

		#Change name process
		scenePG = bpy.context.scene.polygroups.get(storedName)

		if scenePG is not None:

			polyProps = addonPG.listenedParent.object.pgprops
			polyProps.name = newName
			scenePG.name = newName
			addonPG.listenedParent.name = newName


def postUpdateHandler(scene):
	
	global pendingDestuctionPolyGroups, allowUpdate, stored_presels, addonListenedPolygroups

	if allowUpdate is True:

		pendingDestruction = []

		scenePolyGroups = bpy.context.scene.polygroups

		#Listen to stored polygroups and check if a name has changed
		addonPGsLen = len(addonListenedPolygroups)
		if addonPGsLen > 0:
			for j in range(addonPGsLen):

				addonPG = addonListenedPolygroups[j]

				#Check name of the current Group
				checkGroupNameChange(addonPG)



		
		scenePGLen = len(scenePolyGroups)

		if scenePGLen > 0:
			for j in range(scenePGLen):

				groupObj = bpy.data.objects.get(scenePolyGroups[j].name)
				
				if groupObj is not None:

					polyProp = groupObj.pgprops

					if isDeleted(groupObj):
						
						pendingDestuctionPolyGroups.append(PendingDestructionPolyGroup(groupObj, DestructionMode.DESTROY))
					
					else:
						
						childrenLen = len(polyProp.children)
						
						pendingDeletion = []
						
						ndel = 0

						if childrenLen > 0:
							
							for k in range(childrenLen):
								item = polyProp.children[k]

								if item.name in bpy.data.objects:
									obj = bpy.data.objects[item.name]
									
									#check if object was deleted
									if isDeleted(obj):
										
										pendingDeletion.append(k)
										ndel += 1
										
									else:
										
										#Object name has changed
										if obj.name != item.name:
											item.name = obj.name
											
								else:
									pendingDeletion.append(k)
									ndel += 1
						
						#If there are children that have been deleted
						if ndel > 0:
							
							#Remove UIList items
							for x in reversed(pendingDeletion):
								print("Delete " + str(x))
								polyProp.children.remove(x)
								
							#Prevent the list index to be out of range
							max = len(polyProp.children) - 1
							if bpy.context.scene.list_1_index > max:
								bpy.context.scene.list_1_index = max
								
							if len(polyProp.children) < 2:
								
								pendingDestuctionPolyGroups.append(PendingDestructionPolyGroup(groupObj, DestructionMode.DESTROY))
								
						
							else:
								
								#Set the new number of children to the PolyGroup

								#Note: calling property "polyProp.npoints" here will not call correctly the callback because "context.object is NoneType object".
								#The "remakePolyGroup()" function should be called after updating the property
								polyProp.npoints = childrenLen - ndel

								#Call remakePolyGroup() after the property is updated
								remakePolyGroup(polyProp)

								return None
		
		checkDestruction()


	checkListSelection()
	

		
def preFrame(scene):

	scene = bpy.context.scene

	if bpy.context.scene.gotpolygroups:
		for pgName in bpy.context.scene.polygroups:
			pgObject = bpy.data.objects.get(pgName.name)
			if pgObject is not None:
				pgProps = pgObject.pgprops

				remakePolyGroup(pgProps)

	#print("preframe") #debug

def preSave(scene):

	return None

def register():

	bpy.utils.register_class(SelectChildrenOperator)
	bpy.utils.register_class(ToZeroOperator)
	bpy.utils.register_class(GenerateOperator)
	bpy.utils.register_class(CancelOperator)
	bpy.utils.register_class(PropLayoutPanel)
	bpy.utils.register_class(UIList1)
	bpy.utils.register_class(DestroyOperator)
	bpy.utils.register_class(ToggleMaterialOperator)
	bpy.utils.register_class(ToggleExtend0Operator)
	bpy.utils.register_class(ToggleExtend1Operator)
	bpy.utils.register_class(ToggleExtend2Operator)
	bpy.utils.register_class(ResetScaleOperator)
	bpy.utils.register_class(ResetRotationOperator)
	bpy.utils.register_class(PolyGroupChildrenPropertyGroup)
	bpy.utils.register_class(PolyGroupMaterialPropertyGroup)
	bpy.utils.register_class(PolyGroupPropertyGroup)
	bpy.utils.register_class(SelectedChildrenPropertyGroup)
	bpy.utils.register_class(SelectedPropertyGroup)
	bpy.utils.register_class(PolyGroupObjectPropertyGroup)

	
	#bpy.types.Scene.my_tool = PointerProperty(type=MySettings)
	bpy.types.Scene.destination = bpy.props.StringProperty(  
		name = "Destination", 
		subtype = "FILE_PATH", 
		description = "First sequence file location" 
		)
	bpy.types.Scene.radius = bpy.props.FloatProperty(
		name = "Radius",
		description = "A float property",
		default = 2.0,
		min = 0.0,
		max = 999.0
		)
	bpy.types.Scene.npoints_prev = bpy.props.IntProperty(
		name = "Number of points",
		description = "A int property",
		default = 4,
		min = 2,
		max = 100
		)
	bpy.types.Scene.npoints = bpy.props.IntProperty(
		name = "Number of points",
		description = "A int property",
		default = 4,
		min = 2,
		max = 100
		)
	bpy.types.Scene.startangle = bpy.props.FloatProperty(
		name = "Start Angle (deg)",
		description = "The start angle of the polygroup",
		default = 0.0,
		min = -3600.0,
		max = 3600.0
		)
	bpy.types.Scene.endangle = bpy.props.FloatProperty(
		name = "End Angle (deg)",
		description = "The end angle of the polygroup",
		default = 360.0,
		min = 0.0,
		max = 360.0
		)
	bpy.types.Scene.angularoffset = bpy.props.FloatProperty(
		name = "Offset (-1, 1)",
		description = "The angular offset",
		default = 0,
		min = -1.0,
		max = 1.0
		)
	bpy.types.Scene.setconstraints = bpy.props.BoolProperty(
		name = "Set Transform constraints",
		description = "A bool property",
		default = True
		)
	bpy.types.Scene.listselected = bpy.props.BoolProperty(
		name = "List selected",
		description = "A bool property",
		default = False
		)
	bpy.types.Scene.extend_0 = bpy.props.BoolProperty(
		name = "Extend",
		description = "A bool property",
		default = False
		)
	bpy.types.Scene.extend_1 = bpy.props.BoolProperty(
		name = "Extend",
		description = "A bool property",
		default = False
		)
	bpy.types.Scene.extend_2 = bpy.props.BoolProperty(
		name = "Extend",
		description = "A bool property",
		default = False
		)
	bpy.types.Scene.materialinput = bpy.props.StringProperty(
		name = "Material", 
		description = "Group material name",
		update = updateMatInput,
		default = ""
		)
	bpy.types.Scene.tracking = bpy.props.BoolProperty(
		name = "Tracking state",
		description = "A bool property",
		default = False
		)
	bpy.types.Scene.trackingface = bpy.props.EnumProperty(
		items= {
			("0", "X", TRACKENUMSTR + "X"),
			("1", "Y", TRACKENUMSTR + "Y"),
			("2", "-X", TRACKENUMSTR + "-X"),
			("3", "-Y", TRACKENUMSTR + "-Y")
			},  
		default = "0",
		name = "Tracking Axis",
		update = updateTrackingFace
		)
	bpy.types.Scene.trackingfaceangle = bpy.props.FloatProperty(
		name = "Tracking Face angle",
		description = "The angle of face tracking center",
		default = 0.0,
		min = 0.0,
		max = math.radians(360.0)
		)
	bpy.types.Scene.gotpolygroups = bpy.props.BoolProperty(
		name = "Scene has got polygroups",
		description = "A bool property",
		default = False
		)
	bpy.types.Scene.polygroups = bpy.props.CollectionProperty(type = PolyGroupPropertyGroup)
	bpy.types.Scene.selectedpolygroup = bpy.props.CollectionProperty(type = SelectedPropertyGroup)

	bpy.utils.register_class(ListAItem)
	bpy.types.Scene.list_1 = bpy.props.CollectionProperty(type = ListAItem)
	bpy.types.Scene.list_1_index = bpy.props.IntProperty(
		name = "Index for list_1", 
		min = 0, 
		default = 0,
		update = updateObjListIdx
		)
	

	#Object specific properties
	bpy.types.Object.pgprops = bpy.props.PointerProperty(type = PolyGroupObjectPropertyGroup)

	#Reset existing handlers if Add-on as been reloaded

	handlers = bpy.app.handlers
	scene_update_pre = handlers.scene_update_pre
	if len(scene_update_pre) > 0:
		scene_update_pre.clear()

	scene_update_post = handlers.scene_update_post
	if len(scene_update_post) > 0:
		scene_update_post.clear()
	
	frame_change_pre = handlers.frame_change_pre
	if len(frame_change_pre) > 0:
		frame_change_pre.clear()

	save_pre = bpy.app.handlers.save_pre
	if len(save_pre) > 0:
		save_pre.clear()

	scene_update_pre.append(preUpdateHandler)
	scene_update_post.append(postUpdateHandler)
	frame_change_pre.append(preFrame)
	save_pre.append(preSave)

	#init if refresh add-on
	coll = bpy.context.scene.list_1
	coll.clear()
	bpy.context.scene.list_1_index = 0

	bpy.context.scene.polygroups.clear()
	
	bpy.context.scene.listselected = False
	bpy.context.scene.tracking = False
	bpy.context.scene.extend_0 = False
	bpy.context.scene.extend_1 = False
	bpy.context.scene.extend_2 = False
	bpy.context.scene.gotpolygroups = False



def unregister():
	
	bpy.utils.unregister_class(SelectChildrenOperator)
	bpy.utils.unregister_class(ToZeroOperator)
	bpy.utils.unregister_class(GenerateOperator)
	bpy.utils.unregister_class(CancelOperator)
	bpy.utils.unregister_class(PropLayoutPanel)
	bpy.utils.unregister_class(UIList1)
	bpy.utils.unregister_class(DestroyOperator)
	bpy.utils.unregister_class(ToggleMaterialOperator)
	bpy.utils.unregister_class(ToggleExtend0Operator)
	bpy.utils.unregister_class(ToggleExtend1Operator)
	bpy.utils.unregister_class(ToggleExtend1Operator)
	bpy.utils.unregister_class(ResetScaleOperator)
	bpy.utils.unregister_class(ResetRotationOperator)
	bpy.utils.unregister_class(PolyGroupPropertyGroup)
	bpy.utils.unregister_class(PolyGroupChildrenPropertyGroup)
	bpy.utils.unregister_class(PolyGroupMaterialPropertyGroup)
	bpy.utils.unregister_class(SelectedChildrenPropertyGroup)
	bpy.utils.unregister_class(SelectedPropertyGroup)
	bpy.utils.unregister_class(PolyGroupObjectPropertyGroup)
	

	
	#init if refresh add-on
	coll = bpy.context.scene.list_1
	coll.clear()
	
	del bpy.types.Scene.destination
	del bpy.types.Scene.radius
	del bpy.types.Scene.npoints_prev
	del bpy.types.Scene.npoints
	del bpy.types.Scene.startangle
	del bpy.types.Scene.endangle
	del bpy.types.Scene.polygroups
	del bpy.types.Scene.list_1
	del bpy.types.Scene.list_1_index
	del bpy.types.Scene.setconstraints
	del bpy.types.Scene.listselected
	del bpy.types.Scene.selectedpolygroup
	del bpy.types.Scene.extend_0
	del bpy.types.Scene.extend_1
	del bpy.types.Scene.materialinput
	del bpy.types.Scene.tracking
	del bpy.types.Scene.trackingface
	del bpy.types.Scene.gotpolygroups
	del bpy.types.Scene.angularoffset

	del bpy.types.Object.pgprops
	
	bpy.app.handlers.scene_update_pre.remove(preUpdateHandler)
	bpy.app.handlers.scene_update_post.remove(postUpdateHandler)
	bpy.app.handlers.frame_change_pre.remove(preFrame)
	bpy.app.handlers.save_pre.remove(preSave)



if __name__ == "__main__":
	register()
	
#unregister()


